--*******************************************************
-- Demo: DB2-XML on DB2 Version 10.1
-- Author: Dr. Can Trker
--*******************************************************

db2 "--- Setting up the database XMLDB ..."

db2 "CREATE DATABASE XMLDB"

db2 "CONNECT TO XMLDB"

db2 "--- Creating XML schema ..."

db2 "REGISTER XMLSCHEMA 'book.xsd' FROM 'file://C:\XMLDIR\book.xsd' AS book.xsd COMPLETE"

db2 "SELECT OBJECTSCHEMA, OBJECTNAME FROM SYSCAT.XSROBJECTS"

db2 "--- Creating table with XML column ..."

db2 "CREATE TABLE book (Id INT NOT NULL PRIMARY KEY, content XML)"

db2 "--- Load XML data into table ..."

db2 "IMPORT FROM 'C:\XMLDIR\book.del' OF DEL XML FROM C:\XMLDIR XMLVALIDATE USING XDS DEFAULT book.xsd INSERT INTO book"

db2 "--- Querying XML data ..."

db2 "SELECT * FROM book"

db2 "SELECT XMLQUERY('<book Year=\"{$i//@year}\"><title>{$i//title/text()}</title><authors>{for $a in $i//author return <author>{$a/text()}</author>}</authors></book>' PASSING content AS \"i\") AS book FROM book b WHERE XMLEXISTS ('$i/book[author[1] = \"Gunter Saake\"]' PASSING content AS \"i\")"

db2 "SELECT t.* FROM book, XMLTABLE('$i' PASSING content AS \"i\" COLUMNS ISBNNummer VARCHAR(30) PATH '/book/@ISBN', booktitle VARCHAR(50) PATH '/book/title') AS t"

db2 "SELECT XMLQuery('$i//title/text()' PASSING content AS \"i\") AS booktitle FROM book WHERE XMLEXISTS('$i//author[text() = \"Gunter Saake\"]' PASSING content AS \"i\")"

db2 "SELECT XMLCAST(XMLQUERY('$i//book/title' PASSING content AS \"i\") AS CLOB) AS booktitle FROM book"

db2 "--- Creating and using XML index ..."

db2 "CREATE INDEX xmlbooktitleidx ON book (content) GENERATE KEY USING XMLPATTERN '/book/title' AS SQL VARCHAR(50)"

db2 "SELECT XMLQUERY('$i/book/title' PASSING content AS \"i\") AS booktitle FROM book WHERE XMLEXISTS('$i/book/title' PASSING content AS \"i\")"

db2 "--- Creating and using full-text index on XML data (requires enabling the text extender for database XMLDB) ..."

set DB2DBDFT=XMLDB

db2ts "ENABLE DATABASE FOR TEXT CONNECT TO XMLDB"

db2ts "START FOR TEXT"

db2ts "CREATE INDEX xmlfulltextidx FOR TEXT ON book(content)"

db2ts "UPDATE INDEX xmlfulltextidx FOR TEXT"

db2 "SELECT id, INTEGER(SCORE(content, 'XML')*1000) AS score FROM book WHERE CONTAINS(content, 'XML')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, '(XML~0.3)')*1000) AS score FROM book WHERE CONTAINS(content, '(XML~0.3)')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, 'XML Buch')*1000) AS score FROM book WHERE CONTAINS(content, 'XML Buch')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, '(XML AND Buch)')*1000) AS score FROM book WHERE CONTAINS(content, '(XML AND Buch)')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, '(XML OR Buch)')*1000) AS score FROM book WHERE CONTAINS(content, '(XML OR Buch)')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, '(\"XML Buch\")')*1000) AS score FROM book WHERE CONTAINS(content, '(\"XML Buch\")')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, '(\"XML Schema\")')*1000) AS score FROM book WHERE CONTAINS(content, '(\"XML Schema\")')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, '(\"XML\")')*1000) AS score FROM book WHERE CONTAINS(content, '(\"XML\")')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, '@xmlxp:''/book[@year = 2003]''')*1000) AS score FROM book WHERE CONTAINS(content, '@xmlxp:''/book[@year = 2003]''')=1 ORDER BY score DESC"

db2 "SELECT id, INTEGER(SCORE(content, '@xmlxp:''/book/description''')*1000) AS score FROM book WHERE CONTAINS(content, '@xmlxp:''/book/description''')=1 ORDER BY score DESC"

db2 "XQUERY db2-fn:xmlcolumn-contains('book.content',  '@xmlxp:''/book/description''')//title/text()"

db2 "XQUERY for $b in db2-fn:xmlcolumn('book.content')//book where $b/@year > 2000 return $b//author"

db2 "XQUERY db2-fn:sqlquery('SELECT content FROM book')//book[@year > 2000]/author"

db2 "SELECT XMLQUERY('$i//book[@year > 2000]/author' PASSING content as \"i\") FROM book"

db2 "--- Updating XML data ..."

db2 "SELECT XMLQUERY('transform copy $pi := $i modify do insert document { <author>Jim Bob</author> } before $pi//book/author[1] return $pi' PASSING content AS \"i\") FROM book WHERE XMLEXISTS('$i/book[author/text() = \"Holger Meyer\"]' PASSING content AS \"i\")"

db2 "UPDATE book SET content = XMLQUERY('transform copy $pi := $i modify do insert document { <author>Jim Bob</author> } before $pi//book/author[1] return $pi' PASSING content AS \"i\") WHERE XMLEXISTS('$i/book[author/text() = \"Holger Meyer\"]' PASSING content AS \"i\")"

db2 "--- Cleaning up ..."

db2ts "DROP INDEX xmlfulltextidx FOR TEXT"

db2ts "STOP FOR TEXT"

db2ts "DISABLE DATABASE FOR TEXT CONNECT TO XMLDB"

db2 "DROP XSROBJECT book.xsd"

db2 "DROP TABLE book"

db2 "DISCONNECT XMLDB"

db2 "DROP DATABASE XMLDB"

