--*******************************************************
-- Demo: DB2-XML on DB2 Version 10.1
-- Author: Dr. Can Trker
--*******************************************************
-- CALL SYSPROC.SYSTS_DROP('XMLDB', 'xmlfulltextidx', 'en_US, ?);

-- DROP TABLE book;

--*******************************************************
-- Model-based storage of XML documents
--*******************************************************
-- Register an XML schema and create a table that contains an XML column
--*******************************************************

REGISTER XMLSCHEMA 'book.xsd' FROM 'file://C:\XMLDIR\book.xsd' AS book.xsd COMPLETE;

CREATE TABLE book 
(
  Id INT NOT NULL PRIMARY KEY,
  content XML 
);

--*******************************************************
-- Load (schema-validated) XML documents into the database
--*******************************************************

IMPORT FROM 'C:\XMLDIR\book.del' OF DEL XML FROM C:\XMLDIR XMLVALIDATE USING XDS DEFAULT book.xsd INSERT INTO book;

SELECT * FROM book;

--*******************************************************
-- Example queries using the SQL/XML functions
--*******************************************************

COLUMN book FORMAT a78;
COLUMN booktitle FORMAT a45;
COLUMN isbn FORMAT a20;

SELECT XMLQUERY('<Book Year="{$i//@year}"><Title>{$i//Titel/text()}</Title><Authors>{for $a in $i//author return <Author>{$a/text()}</Author>}</Authors></Book>' PASSING content AS "i") AS book
FROM book b
WHERE XMLEXISTS ('$i/book[author[1] = "Gunter Saake"]' PASSING content AS "i");

SELECT t.*
FROM book, XMLTABLE('$i' PASSING content AS "i" COLUMNS
	isbn VARCHAR(30) PATH '/book/@ISBN', 
	booktitle VARCHAR(50) PATH '/book/Titel') AS t;

SELECT XMLQuery('$i//Titel/text()' PASSING content AS "i") AS booktitle
FROM book
WHERE XMLEXISTS('$i//author[text() = "Gunter Saake"]' PASSING content AS "i");

SELECT XMLCAST(XMLQUERY('$i//book/Titel' PASSING content AS "i") AS CLOB) AS booktitle
FROM book;

CREATE TABLE Employee (
   Name VARCHAR(50),
   Salary INTEGER
);

INSERT INTO Employee VALUES ('Joe', 2000);
INSERT INTO Employee VALUES ('Jim', 3500);

SELECT name, XMLROW(name, salary) AS income
FROM employee;

SELECT name, XMLGROUP(salary) AS income
FROM employee
GROUP BY name;

--*******************************************************
-- Create and use an XML index
--*******************************************************

CREATE INDEX xmlbooktitleidx ON book (content)
GENERATE KEY USING XMLPATTERN '/book/Titel' AS SQL VARCHAR(50);

SELECT XMLQUERY('$i/book/Titel' PASSING content AS "i") AS booktitle 
FROM book 
WHERE XMLEXISTS('$i/book/Titel' PASSING content AS "i");

--*******************************************************
-- Create and use a full-text index
--*******************************************************

-- CALL SYSPROC.SYSTS_CREATE('XMLDB', 'xmlfulltextidx', 'book(content)', 'en_US, ?);

SELECT id, INTEGER(SCORE(content, '(XML~0.3)')*1000) AS score FROM book WHERE CONTAINS(content, '(XML~0.3)')=1 ORDER BY score DESC;

SELECT id, INTEGER(SCORE(content, 'XML Buch')*1000) AS score FROM book WHERE CONTAINS(content, 'XML book')=1 ORDER BY score DESC;

SELECT id, INTEGER(SCORE(content, '(XML AND Buch)')*1000) AS score FROM book WHERE CONTAINS(content, '(XML AND book)')=1 ORDER BY score DESC;

SELECT id, INTEGER(SCORE(content, '(XML OR Buch)')*1000) AS score FROM book WHERE CONTAINS(content, '(XML OR book)')=1 ORDER BY score DESC;

SELECT id, INTEGER(SCORE(content, '("XML Buch")')*1000) AS score FROM book WHERE CONTAINS(content, '("XML book")')=1 ORDER BY score DESC;

SELECT id, INTEGER(SCORE(content, '("XML Schema")')*1000) AS score FROM book WHERE CONTAINS(content, '("XML Schema")')=1 ORDER BY score DESC;

SELECT id, INTEGER(SCORE(content, '("XML")')*1000) AS score FROM book WHERE CONTAINS(content, '("XML")')=1 ORDER BY score DESC;

SELECT id, INTEGER(SCORE(content, '@xmlxp:''/book[@year = 2003]''')*1000) AS score FROM book WHERE CONTAINS(content, '@xmlxp:''/book[@year = 2003]''')=1 ORDER BY score DESC;

SELECT id, INTEGER(SCORE(content, '@xmlxp:''/book/description''')*1000) AS score FROM book WHERE CONTAINS(content, '@xmlxp:''/book/description''')=1 ORDER BY score DESC;

XQUERY db2-fn:xmlcolumn-contains('book.content',  '@xmlxp:''/book/description''')//Titel/text();

--*******************************************************
-- XQuery on SQL queries that return a table with only 
-- one column, which above that is from typ xml
--*******************************************************
XQUERY for $b in db2-fn:xmlcolumn('book.content')//book where $b/@year > 2000 return $b//author;

XQUERY db2-fn:sqlquery('SELECT content FROM book')//book[@year > 2000]/author;

--*******************************************************
-- XQuery and XMLQuery are different things! See result
-- of next query which return 5 instead of 4 rows
--*******************************************************
SELECT XMLQUERY('$i//book[@year > 2000]/author' PASSING content as "i") FROM book;

--*******************************************************
-- XQuery Update
--*******************************************************
SELECT XMLQUERY('transform copy $pi := $i modify do insert document { <author>Jim Bob</author> } before $pi//book/author[1] return $pi' PASSING content AS "i") 
FROM book 
WHERE XMLEXISTS('$i/book[author/text() = "Holger Meyer"]' PASSING content AS "i");

UPDATE book 
SET content = XMLQUERY('transform copy $pi := $i modify do insert document { <author>Jim Bob</author> } before $pi//book/author[1] return $pi' PASSING content AS "i")
WHERE XMLEXISTS('$i/book[author/text() = "Holger Meyer"]' PASSING content AS "i");

