--*******************************************************
-- Demo: MSSQL-XML 
-- Author: Dr. Can Trker
--*******************************************************

--DROP TABLE xmltable;

CREATE TABLE xmltable (
  docid   BIGINT IDENTITY(1,1) PRIMARY KEY,
  doctext VARCHAR(8000)
);

SELECT * FROM xmltable;

INSERT INTO xmltable(doctext) VALUES ('<?xml version="1.0" encoding="ISO-8859-1"?><book isbn="3-89864-148-1" year="2003"><author>Meike Klettke</author><author>Holger Meyer</author><title>XML &amp; Datenbanken</title><subtitle>XML-Dokumente effizient speichern und verarbeiten</subtitle><publisher city="Heidelberg">dpunkt</publisher><description>Mit der wachsenden Verbreitung von XML als Format zur Speicherung und zum Austausch von Daten werden auch neue Anforderungen an Datenbanksysteme gestellt. Dieses Buch beschreibt detailliert das Zusammenwirken von XML und Datenbanken und zeigt, welche neuen Konzepte, Sprachen und Systeme dabei entstanden und noch entstehen. Schwerpunkte sind dabei die Speicherung von XML-Dokumenten, Indizierungsverfahren, XML-Anfragesprachen, Verfahren zur konzeptuellen Modellierung sowie die Erzeugung von XML-Dokumenten aus herkmmlichen Datenbanken. Zu jedem Thema werden Forschungsanstze sowie praktisch eingesetzte Verfahren vorgestellt und klassifiziert. Insbesondere werden behandelt: XML-Grundlagen, DTDs und XML Schema, DOM und SAX, Architektur von XML-Anwendungen, Speicherung von XML-Dokumenten, Indizierungsverfahren und XPath/XQuery. Es wird ausserdem gezeigt, wie XML-Dokumente mit relationalen und objektrelationalen Datenbanksystemen (IBM DB2, Oracle und MS SQL Server) verarbeitet werden knnen und welche speziellen XML-Datenbanksysteme bereits existieren (u.a. eXcelon XIS, Tamino, infonyte).</description></book>');

SELECT * FROM xmltable;

--*******************************************************
-- Creating transient EDGE table
--*******************************************************
DECLARE @xmldoctext VARCHAR(8000) = '<?xml version="1.0" encoding="ISO-8859-1"?><book isbn="3-89864-148-1" year="2003"><author>Meike Klettke</author><author>Holger Meyer</author><title>XML &amp; Datenbanken</title><subtitle>XML-Dokumente effizient speichern und verarbeiten</subtitle><publisher city="Heidelberg">dpunkt</publisher><description>Mit der wachsenden Verbreitung von XML als Format zur Speicherung und zum Austausch von Daten werden auch neue Anforderungen an Datenbanksysteme gestellt. Dieses Buch beschreibt detailliert das Zusammenwirken von XML und Datenbanken und zeigt, welche neuen Konzepte, Sprachen und Systeme dabei entstanden und noch entstehen. Schwerpunkte sind dabei die Speicherung von XML-Dokumenten, Indizierungsverfahren, XML-Anfragesprachen, Verfahren zur konzeptuellen Modellierung sowie die Erzeugung von XML-Dokumenten aus herkmmlichen Datenbanken. Zu jedem Thema werden Forschungsanstze sowie praktisch eingesetzte Verfahren vorgestellt und klassifiziert. Insbesondere werden behandelt: XML-Grundlagen, DTDs und XML Schema, DOM und SAX, Architektur von XML-Anwendungen, Speicherung von XML-Dokumenten, Indizierungsverfahren und XPath/XQuery. Es wird ausserdem gezeigt, wie XML-Dokumente mit relationalen und objektrelationalen Datenbanksystemen (IBM DB2, Oracle und MS SQL Server) verarbeitet werden knnen und welche speziellen XML-Datenbanksysteme bereits existieren (u.a. eXcelon XIS, Tamino, infonyte).</description></book>',

DECLARE @hdoc INT;
 
EXEC sp_xml_preparedocument @hdoc OUTPUT, @xmldoctext;

SELECT * FROM OPENXML(@hdoc, '', 0);

EXEC sp_xml_removedocument @hdoc;

--*******************************************************
-- Creating and filling persistent EDGE table
--*******************************************************
DECLARE @xmldoctext VARCHAR(8000);
DECLARE @hdoc INT;

SELECT @xmldoctext = doctext FROM xmltable WHERE docid = 1;

--DROP TABLE edge;
CREATE TABLE edge (
  id	        BIGINT PRIMARY KEY,
  parentid	BIGINT,
  nodetype	INT,
  localname	NVARCHAR(4000),
  prefix	NVARCHAR(4000),
  namespaceuri	NVARCHAR(4000),
  datatype 	NVARCHAR(4000),
  prev		BIGINT,
  text		NTEXT
);

EXEC sp_xml_preparedocument @hdoc OUTPUT, @xmldoctext;
INSERT INTO EDGE 
SELECT *
FROM OPENXML (@hdoc, '', 0); 
EXEC sp_xml_removedocument @hdoc;

SELECT * FROM edge;

--*******************************************************
-- Structure mapping results in transient tables
--*******************************************************
DECLARE @xmldoctext VARCHAR(8000) = '<?xml version="1.0" encoding="ISO-8859-1"?><book isbn="3-89864-148-1" year="2003"><author>Meike Klettke</author><author>Holger Meyer</author><title>XML &amp; Datenbanken</title><subtitle>XML-Dokumente effizient speichern und verarbeiten</subtitle><publisher city="Heidelberg">dpunkt</publisher><description>Mit der wachsenden Verbreitung von XML als Format zur Speicherung und zum Austausch von Daten werden auch neue Anforderungen an Datenbanksysteme gestellt. Dieses Buch beschreibt detailliert das Zusammenwirken von XML und Datenbanken und zeigt, welche neuen Konzepte, Sprachen und Systeme dabei entstanden und noch entstehen. Schwerpunkte sind dabei die Speicherung von XML-Dokumenten, Indizierungsverfahren, XML-Anfragesprachen, Verfahren zur konzeptuellen Modellierung sowie die Erzeugung von XML-Dokumenten aus herkmmlichen Datenbanken. Zu jedem Thema werden Forschungsanstze sowie praktisch eingesetzte Verfahren vorgestellt und klassifiziert. Insbesondere werden behandelt: XML-Grundlagen, DTDs und XML Schema, DOM und SAX, Architektur von XML-Anwendungen, Speicherung von XML-Dokumenten, Indizierungsverfahren und XPath/XQuery. Es wird ausserdem gezeigt, wie XML-Dokumente mit relationalen und objektrelationalen Datenbanksystemen (IBM DB2, Oracle und MS SQL Server) verarbeitet werden knnen und welche speziellen XML-Datenbanksysteme bereits existieren (u.a. eXcelon XIS, Tamino, infonyte).</description></book>';

DECLARE @hdoc INT;

EXEC sp_xml_preparedocument @hdoc OUTPUT, @xmldoctext;

SELECT * FROM OPENXML (@hdoc, '//book', 0) WITH (	
	title  NVARCHAR(3000) './title',
	isbn   NVARCHAR(15)   './@isbn',
	publisher NVARCHAR(200)  './publisher');

SELECT * FROM OPENXML (@hdoc, '//publisher', 0) WITH (	
	name NVARCHAR(200) '.',
	city  NVARCHAR(200) './@city');

SELECT * FROM OPENXML (@hdoc, '//author', 0) WITH (	
	name  NVARCHAR(200) '.',
	isbn  NVARCHAR(200) '../@isbn');

EXEC sp_xml_removedocument @hdoc;

--*******************************************************
-- Structure mapping results in persistent tables
--*******************************************************
DECLARE @xmldoctext VARCHAR(8000);
DECLARE @hdoc INT;

SELECT @xmldoctext = doctext FROM xmltable WHERE docid = 1;

--DROP TABLE publisher;
--DROP TABLE author;
--DROP TABLE book;

CREATE TABLE book (
  id		BIGINT IDENTITY(1,1) PRIMARY KEY,
  title		NVARCHAR(3000),
  isbn		NVARCHAR(15) UNIQUE,
  publisher	NVARCHAR(200)
); 

CREATE TABLE author (
  name 	NVARCHAR(200),
  isbn	NVARCHAR(15) FOREIGN KEY REFERENCES book(isbn)
); 

CREATE TABLE publisher (
  name 	NVARCHAR(200),
  city	NVARCHAR(200)
); 

EXEC sp_xml_preparedocument @hdoc OUTPUT, @xmldoctext;

INSERT INTO book (title, isbn, publisher)
SELECT * FROM OPENXML (@hdoc, '//book', 0) WITH (	
	title  NVARCHAR(3000) './title',
	isbn   NVARCHAR(15)   './@isbn',
	publisher NVARCHAR(200)  './publisher');

INSERT INTO author (name, isbn)
SELECT * FROM OPENXML (@hdoc, '//author', 0) WITH (	
	name  NVARCHAR(200) '.',
	isbn  NVARCHAR(15) '../@isbn');

INSERT INTO publisher (name, city)
SELECT * FROM OPENXML (@hdoc, '//publisher', 0) WITH (	
	name NVARCHAR(200) '.',
	city  NVARCHAR(200) './@city');

EXEC sp_xml_removedocument @hdoc;

SELECT * FROM book;
SELECT * FROM author;
SELECT * FROM publisher;

--*******************************************************
-- Publishing tables in xml format
--*******************************************************
SELECT name, title, publisher, book.isbn
FROM author JOIN book ON (author.isbn = book.isbn)
FOR XML AUTO;

SELECT name, title, publisher, book.isbn
FROM author JOIN book ON (author.isbn = book.isbn)
FOR XML AUTO, ELEMENTS;

SELECT name, title, publisher, book.isbn
FROM author JOIN book ON (author.isbn = book.isbn)
FOR XML AUTO, ELEMENTS, XMLSCHEMA ('MyBook');

SELECT name, title, publisher, book.isbn
FROM author JOIN book ON (author.isbn = book.isbn)
FOR XML RAW;

SELECT name, title, publisher, book.isbn
FROM author JOIN book ON (author.isbn = book.isbn)
FOR XML RAW, ELEMENTS;

SELECT book.isbn AS "@isbn", name AS "author", publisher AS "title/@publisher", title AS "title/text()"
FROM author JOIN book ON (author.isbn = book.isbn)
FOR XML PATH;

SELECT book.isbn AS "@isbn", name AS "author", publisher AS "title/@publisher", title AS "title/text()"
FROM author JOIN book ON (author.isbn = book.isbn)
FOR XML PATH ('books'), ROOT ('bookstore');

