--*******************************************************
-- Demo: Oracle-XML 
-- author: Dr. Can Trker
--*******************************************************

SET FEEDBACK 1;
--SET NUMWIDTH 50;
--SET TRIMSPOOL ON;
--SET TAB OFF;
--SET PAGESIZE 100;
SET LINESIZE 80;
SET serveroutput ON;
SET termout ON;
SET flush ON;
SET ECHO ON;
SET LONG 8000;

--*******************************************************
-- CLOB storage of XML documents
--*******************************************************

CREATE TABLE bookxml (
  content XMLTYPE
);

--*******************************************************
-- Load XML documents into the database
--*******************************************************

INSERT INTO bookxml VALUES (XMLTYPE(getDocument('book1.xml')));
INSERT INTO bookxml VALUES (XMLTYPE(getDocument('book2.xml')));
INSERT INTO bookxml VALUES (XMLTYPE(getDocument('book3.xml')));
INSERT INTO bookxml VALUES (XMLTYPE(getDocument('book4.xml')));

--*******************************************************
-- Example queries using the SQL/XML functions
--*******************************************************

COLUMN book FORMAT a78;
COLUMN booktitle FORMAT a45;
COLUMN isbn FORMAT a20;

SELECT XMLQUERY('<book year="{$i//@year}"> 
                   <title>{$i//title/text()}</title>
                   <authors>{for $a in $i//author return <author>{$a/text()}</author>}</authors>  
                 </book>'                                                                                                                    
                 PASSING content AS "i" RETURNING CONTENT) AS book
FROM bookxml
WHERE XMLQUERY('$i/book[author[1] = "Gunter Saake"]' PASSING content AS "i" RETURNING CONTENT) IS NOT NULL;

SELECT t.*
FROM bookxml, XMLTABLE('$i' PASSING content AS "i" COLUMNS
	isbn VARCHAR(30) PATH '/book/@isbn', 
	booktitle VARCHAR(50) PATH '/book/title') AS t;

SELECT XMLQuery('$i//title/text()' PASSING content AS "i" RETURNING CONTENT) AS booktitle
FROM bookxml
WHERE XMLEXISTS('$i//author[text() = "Gunter Saake"]' PASSING content AS "i");

SELECT XMLCAST(XMLQUERY('$i//book/title' PASSING content AS "i" RETURNING CONTENT) AS CLOB) AS booktitle
FROM bookxml;

--*******************************************************
-- Set AUTOTRACE on EXPLAIN to query execution plan
--*******************************************************
SET AUTOTRACE ON EXPLAIN;

--*******************************************************
-- Create and use a functional index
--*******************************************************

SELECT XMLQUERY('<book title="{$i//title/text()}" year="{$i//@year}"/>' PASSING content AS "i" RETURNING CONTENT) AS book
FROM bookxml
WHERE  XMLCAST(XMLQUERY('$i//@year' PASSING content AS "i" RETURNING CONTENT) AS INTEGER) > 2000;

CREATE INDEX xmlfunktionalidx2 ON bookxml b 
(XMLCAST(XMLQUERY('$i//@year' PASSING content AS "i" RETURNING CONTENT) AS INTEGER));

SELECT XMLQUERY('<book title="{$i//title/text()}" year="{$i//@year}"/>' PASSING content AS "i" RETURNING CONTENT) AS book
FROM bookxml
WHERE  XMLCAST(XMLQUERY('$i//@year' PASSING content AS "i" RETURNING CONTENT) AS INTEGER) > 2000;

--*******************************************************
-- Create and use a full-text index
--*******************************************************
-- Contains function requires a full-text index
--*******************************************************
-- If base data is updated, the index must be update explicitly 
-- using the following command:
-- ALTER INDEX <Name> REBUILD;
--*******************************************************

CREATE INDEX xmlfulltextidx2 ON bookxml b (content) INDEXTYPE IS CTXSYS.CONTEXT;

SELECT XMLQUERY('$i//@isbn' PASSING content AS "i" RETURNING CONTENT) AS isbn, SCORE(0)
FROM bookxml
WHERE CONTAINS(content, 'Das Buch', 0) > 0
ORDER BY SCORE(0) DESC;

--*******************************************************
-- Create and use an XML index
--*******************************************************

SELECT XMLQUERY('$i//book/title' PASSING content AS "i" RETURNING CONTENT) AS booktitle
FROM bookxml
WHERE XMLEXISTS('$i//book/title' PASSING content AS "i");

CREATE INDEX xmlidx2 ON bookxml b 
(content) INDEXTYPE IS XDB.XMLIndex;

SELECT XMLQUERY('$i//book/title' PASSING content AS "i" RETURNING CONTENT) AS booktitle
FROM bookxml
WHERE XMLEXISTS('$i//book/title' PASSING content AS "i");

--*******************************************************
-- Switch off AUTOTRACE
--*******************************************************
SET AUTOTRACE OFF;

--*******************************************************
-- Updates XML elements
--*******************************************************

COLUMN publisher FORMAT a40;
COLUMN publisherscity FORMAT a30;

SELECT XMLQUERY('$i//publisher/text()' PASSING content AS "i" RETURNING CONTENT) AS publisher, XMLQUERY('$i//@city' PASSING content AS "i" RETURNING CONTENT) AS publisherscity
FROM bookxml;

UPDATE bookxml
SET content = UPDATEXML(content, '//publisher[text()="dpunkt"]/@city', 'Zrich');

UPDATE bookxml
SET content = UPDATEXML(content, '//publisher[text()="dpunkt"]/text()', 'dpunkt-Verlag');

SELECT XMLQUERY('$i//publisher/text()' PASSING content AS "i" RETURNING CONTENT) AS publisher, XMLQUERY('$i//@city' PASSING content AS "i" RETURNING CONTENT) AS publisherscity
FROM bookxml;

--*******************************************************
-- Full-Text Index Rebuild
--*******************************************************

ALTER INDEX xmlfulltextidx2 REBUILD;

SELECT XMLQUERY('$i//publisher/text()' PASSING content AS "i" RETURNING CONTENT) AS publisher,
       XMLQUERY('$i//title/text()' PASSING content AS "i" RETURNING CONTENT) AS booktitle
FROM bookxml;

EXIT;
/


