--*******************************************************
-- Demo: PostgreSQL-XML
-- Author: Dr. Can Trker
--*******************************************************

--*******************************************************
-- Function for loading XML data from files
--*******************************************************
CREATE OR REPLACE FUNCTION getDocument(p_filename character varying) RETURNS XML AS
$$SELECT XMLPARSE(DOCUMENT convert_from(pg_read_binary_file(E'XMLDIR/' || $1), 'ISO-8859-1'));$$
LANGUAGE 'sql' VOLATILE SECURITY DEFINER;

--*******************************************************
-- CLOB storage of XML documents
--*******************************************************

CREATE TABLE book (
  content XML
);

--*******************************************************
-- Load XML documents into the database
-- Note: Use the following to get the root directory where the files are read from.
-- SELECT name, setting FROM pg_settings WHERE name='data_directory'; 
--*******************************************************

INSERT INTO book VALUES (getDocument('book1.xml'));
INSERT INTO book VALUES (getDocument('book2.xml'));
INSERT INTO book VALUES (getDocument('book3.xml'));
INSERT INTO book VALUES (getDocument('book4.xml'));

--*******************************************************
-- Example queries using XML funtions
--*******************************************************

SELECT XPATH('//title/text()', content) AS booktitle, XML('<authors>' || TEXT(XPATH('//author', content)) || '</authors>') AS authorsnamen
FROM book
WHERE XMLEXISTS('//author[text() = "Gunter Saake"]' PASSING content);

SELECT XPATH('//author[1]', content) AS firstauthor, TEXT(UNNEST(XPATH('//author[1]', content))) AS firstauthorstschachtelt
FROM book;

SELECT XML('<authors>' || TEXT(XPATH('//author', content)) || '</authors>') AS authorsnamen
FROM book;

SELECT XML('<authors>' || TEXT(UNNEST(XPATH('//author/text()', content))) || '</authors>') AS authorsnamen
FROM book;

SELECT XPATH('//author/text()', content), XPATH('//author[2]/text()', content)
FROM book
WHERE ARRAY_LENGTH(XPATH('//author[2]/text()', content), 1) > 0;

SELECT XPATH('//@isbn', content) AS isbn, XPATH('//title/text()', content) AS title, XPATH('//title', content) AS title2
FROM book;

SELECT t.*
FROM book b, XMLTABLE('*' PASSING content COLUMNS
	isbn VARCHAR(30) PATH '/book/@isbn', 
	booktitle VARCHAR(50) PATH '/book/title') AS t;

SELECT t.*
FROM book b, XMLTABLE('/book' PASSING content COLUMNS
	isbn VARCHAR(30) PATH '@isbn', 
	booktitle VARCHAR(50) PATH 'title') AS t;

SELECT xmltable.*
FROM book b, XMLTABLE('/book' PASSING content COLUMNS
	isbn VARCHAR(30) PATH '@isbn', 
	booktitle VARCHAR(50) PATH 'title');

SELECT XMLELEMENT(NAME "isbn", XPATH('//@isbn', content)) AS XMLvalue
FROM book;

SELECT XMLELEMENT(NAME "firstauthor", XPATH('//author[1]/text()', content)) AS XMLvalue
FROM book;

SELECT XMLELEMENT(NAME "authors", XPATH('//author', content)) AS XMLvalue
FROM book;

SELECT TEXT(UNNEST(XPATH('//author/text()', content))) AS EntschachtelterTextvalue
FROM book;

--*******************************************************
-- Creating and using functional indexes
--*******************************************************

CREATE INDEX xmlfunktionalidx ON book USING BTREE (((XPATH('//author/text()', content))[1]::TEXT));

--EXPLAIN
SELECT XPATH('//title/text()', content) AS booktitle
FROM book
WHERE (XPATH('//author/text()', content))[1]::TEXT = 'Gunter Saake';

--*******************************************************
-- Creating and using full-text indexes
--*******************************************************
CREATE INDEX xmlfulltextidx ON book USING GIN (to_tsvector('german', content::TEXT));

--EXPLAIN
SELECT XPATH('//title/text()', content) AS booktitle
FROM book
WHERE to_tsvector(content::TEXT) @@ plainto_tsquery('german','Gunter');
